<?php
/*
 * Gasell: Copyright 2005-2006 Linux-Nantes Association <contact@gasell.org>
 *
 * You should have received a copy of the GNU Public
 * License along with this package; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Authors: Lionel Corgnet (leon) <leon@gmx.fr>
 *          Damien Raude-Morvan	(DrazziB) <drazzib@drazzib.com>
 *
 * description du fichier gasell/lib/Authentication.php :
 *  ce fichier fourni une bibliotheque de fonctions necessaires ou inutiles
 * a la gestion de l'authentification.
 *
 *
 * Historique :
 *  * 25/06/2006 DrazziB : Utilisation d'une connection Bdd comme attribut de
 *    l'objet.
 *  * 16/06/2005 Leon :
 *    . creation d'une classe (pour utiliser un objet)
 *    . ajout/suppression/renommage de fonctions
 *    . definition de constantes (pour les niveaux d'authentification)
 *  * 19/05/2005 Leon : creation d'une bibliotheque de fonctions
 *
 *
 * TODO :
 *  - gestion de l'inactivite d'une session (timeout)
 */

include_once dirname(__FILE__) . '/../config/conf.php';
require_once dirname(__FILE__) . '/Bdd.php';

/* definition des niveaux d'authentification */
define("D_GSL_AUTH_FAILED", -2); // l'authentification a echoue (login rejete)
define("D_GSL_AUTH_INIT",   -1); // non authentifie (etape initiale)
define("D_GSL_AUTH_REQ",     0); // authentification requise / demandee
define("D_GSL_AUTH_VERIF",   1); // authentification en cours
define("D_GSL_AUTH_OK",      2); // authentifie avec succes

/**
 * @class Authentication
 *
 * @brief permet de gerer l'authentification des utilisateurs
 * dans l'application GASELL.
 * @version $Revision$
 * @package gasell
 */
class Authentication {

	var $step = D_GSL_AUTH_INIT; // niveau d'authentification courant
	var $user = NULL;            // nom d'utilisateur (presque) authentifie
	//var $last_access;            // date de dernier acces
	var $db = NULL;

	/**
	 * @function Authentication::Authentication()
	 *
	 * @brief contructeur de la classe.
	 * @param $from_session vrai si l'on souhaite r�up�er l'objet enregistre en
	 * session, faux sinon
	 * @return l'objet instancie
	 */
	function Authentication ($from_session = true)
	{
		$this->db = new Bdd();

		if ($from_session) {
			$this->sessionRead();
		}

		return $this;
	}


	/**
	 * @function Authentication::setStep()
	 *
	 * @brief modifie l'etape d'authentification actuelle.
	 * @param $step la nouvelle etape d'authentification
	 * @param $init_user vrai s'il faut reinitialiser l'utilisateur
	 * a la valeur par defaut, faux sinon
	 * @return l'etape d'authentification precedente
	 */
	function setStep ($step, $init_user = true)
	{
		$res = $this->step;
		$this->step = $step;

		if ($init_user)
			$this->user = NULL;

		return $res;
	}


	/**
	 * @function Authentication::setRequired()
	 *
	 * @brief indique qu'une authentification est necessaire.
	 * @param $init_user vrai s'il faut reinitialiser l'utilisateur, faux sinon
	 * @return l'etape d'authentification precedente
	 */
	function setRequired ($init_user = false)
	{
		$res = $this->setStep(D_GSL_AUTH_REQ, $init_user);
		return $res;
	}
	

	/**
	 * @function Authentication::setVerifying()
	 *
	 * @brief indique qu'une authentification est en cours.
	 * @param $init_user vrai s'il faut reinitialiser l'utilisateur, faux sinon
	 * @return l'etape d'authentification precedente
	 */
	function setVerifying ($init_user = false)
	{
		$res = $this->setStep(D_GSL_AUTH_VERIF, $init_user);
		return $res;
	}


	/**
	 * @function Authentication::setRefused()
	 *
	 * @brief indique que l'authentification est refusee.
	 * @param $init_user vrai s'il faut reinitialiser l'utilisateur, faux sinon
	 * @return l'etape d'authentification precedente
	 */
	function setRefused ($init_user = true)
	{
		$res = $this->setStep(D_GSL_AUTH_FAILED, $init_user);
		return $res;
	}


	/**
	 * @function Authentication::setProposedUser()
	 *
	 * @brief indique l'identifiant d'un utilisateur en cours d'authentification.
	 * @param $userID l'identifiant fourni
	 */
	function setProposedUser ($userID)
	{
		$this->user = $userID;
		return true;
	}


	/**
	 * @function Authentication::validateUser()
	 *
	 * @brief indique que l'authentification de l'utilisateur est reussie.
	 */
	function validateUser ()
	{
		$res = $this->setStep(D_GSL_AUTH_OK, false);
		//return $res;
	}


	/**
	 * @function Authentication::isOk()
	 *
	 * @brief indique si un utilisateur de l'application est authentifie.
	 * @return vrai si un utilisateur est authentifie, faux sinon.
	 */
	function isOk ()
	{
		return ($this->step == D_GSL_AUTH_OK);
	}


	/**
	 * @function Authentication::getUser()
	 *
	 * @brief fournit l'identifiant de l'utilisateur authentifie.
	 * Il peut etre necessaire de verifier auparavant qu'une authentification
	 * a ete realisee avec succes, a l'aide de la fonction isOk().
	 * @return l'identifiant, ou NULL.
	 */
	function getUser()
	{
		if ($this->isOk()) {
			$res = $this->user;
		}
		else {
			$res = NULL;
		}

		return $res;
	}


	/**
	 * @function Authentication::sessionWrite()
	 *
	 * @brief ecrit l'objet en session
	 * @return vrai si tout s'est bien passe, faux sinon
	 */
	function sessionWrite ()
	{
		$_SESSION['gasell_auth']['step'] = $this->step;
		$_SESSION['gasell_auth']['user'] = $this->user;
		return true;
	}


	/**
	 * @function Authentication::sessionRead
	 *
	 * @brief initialisation a partir de l'objet en session
	 */
	function sessionRead ()
	{
		if (isset($_SESSION['gasell_auth'])) {
			$this->step = $_SESSION['gasell_auth']['step'];
			$this->user = $_SESSION['gasell_auth']['user'];
			$res = true;
		}
		else {
			$res = false;
		}

		return $res;
	}


	/**
	 * @brief verifie le mot de passe d'un pretendu utilisateur.
	 * La verification est effetuee aupres de la base de donnees (ou d'un service
	 * d'authentification autre?..)
	 *
	 * @param $pass mot de passe (non crypte)
	 * @return l'identifiant de l'utilisateur si le couple user/pass existe
	 * (et que c'est le seul mot de passe existant), ou NULL sinon
	 */
	function checkPassword ($pass)
	{
		$result = NULL;

		/* recuperation du mot de passe connu pour l'utilisateur fourni */
		$req = "SELECT user_pass FROM gasell_users";
		$req .= " WHERE user_uid='" . $this->user . "';";
		$res = $this->db->Requette($req, 'ALL');

		/* traitement des information obtenues */
		switch (count($res)) {

		case 0 :
			/* le nom d'utilisateur fourni n'existe pas */
			break;

		case 1 :
			/* le nom fourni est associe a un, et un seul, mot de passe */
			/* que l'on compare a celui fourni */
			if ($res[0][0] == md5($pass)) {
				$result = $this->user;
			}
			break;

		default :
			/* le nom est associe a plusieurs mots de passe : attention, doublons! */
		}
		
		return $result;
	}

}
/* EOF */
?>
