<?php
/*
 * Gasell: Copyright 2004-2006 Linux-Nantes Association <contact@linux-nantes.fr.eu.org>
 *
 * You should have received a copy of the GNU Public
 * License along with this package; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * @doc http://www.gasell.org/wikini/wakka.php?wiki=Exceptions
 *
 * Authors : Poulard Fabien (GrdScarabe) <grdscarabe@grdscarabe.net>
 *           Raude-Morvan Damien (DrazziB) <drazzib@drazzib.com>
 */

/*------------------- CONFIGURATION ----------------------*/
/* on fait un include pour eviter de faire planter le 
   gestionnaire d'erreur si la conf n'est pas trouvee */
@include_once GASELL_BASE . '/config/conf.php';
/*--------------------------------------------------------*/

/** ======================================================= *
 * @class Except
 *
 * @brief classe permettant la gestion des erreurs dans
 * GASELL de maniere centralise/propre et evolutive
 *
 * @noattributes
 *
 * @methodes
 *  - throwGlop(origine, message, data) -> messages de debugage, ne sont
 *  affiches que si $conf['debug_level'] == E_ALL
 *  - throwWarning(origine, message, data) -> messages d'avertissement
 *  - throwError(origine, message, data) -> messages d'erreur provoquant 
 *  l'arret de la methode courante mais permettant la continuation de 
 *  l'execution du script de maniere a peu pres correcte
 *  - throwFailure(origine, message, data) -> tout qui se casse la gueule
 *
 * @version $Revision$
 * @package gasell
 ** ======================================================= */
class Except{

  /**
   * @function throwGlop
   *
   * @brief permet de gerer une erreur permettant le debuggage
   * Ces erreurs ne sont prises en compte que lorsque $conf['debug_level']
   * est place a E_ALL
   *
   * @param origine  chaine decrivant ou l'erreur a eu lieue 
   * (ex: lib/Commissions.php::Commission::loadDB)
   * @param message  est un message d'erreur decrivant l'erreur
   * @param data     est un tableau contenant des donnees propres 
   * a l'erreur ce qui permettra de traiter le tout de maniere plus 
   * precise plus tard sans avoir a tout reprogrammer. Un attribut 
   * obligatoire du tableau etant le numero d'erreur
   **/
  function throwGlop($origine, $message, $data = NULL){
    /* verifie que l'on doit prendre l'erreur en compte */
    if(isset($conf['debug_level']) && ($conf['debug_level'] == E_ALL)){
      echo "<b>$origine</b> a renvoy&eacute; l'erreur de debuggage suivante :";
      echo "<br /><i>$message</i><br />";
	if (isset($data))
	{
      echo "Les donn&eacute;es d'information associ&eacute;es sont :<br />";
      /* on affiche le tableau */
      foreach($data as $key=>$val){
	echo "<b>$key</b> : <i>$val</i> <br />";
      }
	}
    }
  }


  /**
   * @function throwWarning
   *
   * @brief permet de gerer un avertissement
   *
   * @param origine  chaine decrivant ou l'erreur a eu lieue 
   * (ex: lib/Commissions.php::Commission::loadDB)
   * @param message  est un message d'erreur decrivant l'erreur
   * @param data     est un tableau contenant des donnees propres 
   * a l'erreur ce qui permettra de traiter le tout de maniere plus 
   * precise plus tard sans avoir a tout reprogrammer. Un attribut 
   * obligatoire du tableau etant le numero d'erreur
   **/
  function throwWarning($origine, $message, $data = NULL){
    echo "<b>$origine</b> a renvoy&eacute; l'avertissement suivant :";
    echo "<br /><i>$message</i><br />";
	if (isset($data))
	{
    echo "Les donn&eacute;es d'information associ&eacute;es sont :<br />";
    /* on affiche le tableau */
    foreach($data as $key=>$val){
      echo "<b>$key</b> : <i>$val</i> <br />";
    }
	}
  }


  /**
   * @function throwError
   *
   * @brief permet de gerer une erreur recuperable
   *
   * @param origine  chaine decrivant ou l'erreur a eu lieue 
   * (ex: lib/Commissions.php::Commission::loadDB)
   * @param message  est un message d'erreur decrivant l'erreur
   * @param data     est un tableau contenant des donnees propres 
   * a l'erreur ce qui permettra de traiter le tout de maniere plus 
   * precise plus tard sans avoir a tout reprogrammer. Un attribut 
   * obligatoire du tableau etant le numero d'erreur
   **/
  function throwError($origine, $message, $data = NULL){
    /* on utilise le template si on le trouve */
    if(defined('GASELL_TEMPLATES') && file_exists(GASELL_TEMPLATES . '/error.php')) {
      
      /* check if the arguements were passed */
      $retour  = isset($data['back'])       ? $data['back'] : array();
      $address = isset($data['hierarchie']) ? $data['hierarchie'] : array();
      $constat = $message;
      
      /* include the template */
      include GASELL_TEMPLATES . '/error.php';

    } else {
      /* erreur personnalise */
      echo "<b>$origine</b> a renvoy&eacute; l'erreur suivante :";
      echo "<br /><i>$message</i><br />";
	if (isset($data))
	{
      echo "Les donn&eacute;es d'information associ&eacute;es sont :<br />";
      /* on affiche le tableau */
      foreach($data as $key=>$val){
	echo "<b>$key</b> : <i>$val</i> <br />";
      }
	}
    }

    /* on stoppe l'execution */
    exit;
  }


  /**
   * @function throwFailure
   *
   * @brief permet de gerer une erreur irrecuperable
   *
   * @param origine  chaine decrivant ou l'erreur a eu lieue 
   * (ex: lib/Commissions.php::Commission::loadDB)
   * @param message  est un message d'erreur decrivant l'erreur
   * @param data     est un tableau contenant des donnees propres 
   * a l'erreur ce qui permettra de traiter le tout de maniere plus 
   * precise plus tard sans avoir a tout reprogrammer. Un attribut 
   * obligatoire du tableau etant le numero d'erreur
   **/
  function throwFailure($origine, $message, $data = NULL){
    /* on utilise le template si on le trouve */
    if(defined('GASELL_TEMPLATES') && file_exists(GASELL_TEMPLATES . '/erreur.php')) {

      /* check if the arguements were passed */
      $ecode = isset($data['errcode']) ? $data['errcode'] : 0;
      $emsg  = isset($data['errmsg'])  ? $data['errmsg']  : "Heu ...";

      /* Appel du template d'erreur fatale */
      include GASELL_TEMPLATES . '/erreur.php';

    } else {
      /* erreur personnalise */
      echo "<b>$origine</b> a renvoy&eacute; l'erreur fatale suivante :";
      echo "<br /><i>$message</i><br />";
	if (isset($data))
	{
      echo "Les donn&eacute;es d'information associ&eacute;es sont :<br />";
      /* on affiche le tableau */	
      foreach($data as $key=>$val){
	echo "<b>$key</b> : <i>$val</i> <br />";
      }
	}
      echo "L'execution du script va maintenant &ecirc;tre stopp&eacute;e !";
    }

    /* on stoppe l'execution */
    exit;
  }


}

/* -------------------------------------------------------------- *
 *  Exemples d'utilisation :                                      *
 *     Except::throwGlop("ici","glop",array('num'=>1));        *
 *     Except::throwWarning("ici","warning",array('num'=>2));  *
 *     Except::throwError("ici","error",array('num'=>3));      *
 *     Except::throwFailure("ici","failure",array('num'=>4));  *
 * -------------------------------------------------------------- */

/* EOF */
?>