<?php
/*
 * Gasell: Copyright 2004-2006 Linux-Nantes Association <contact@linux-nantes.org>
 *
 * You should have received a copy of the GNU Public
 * License along with this package; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Authors: Saint-Genest Gwenael (Hooligan0) <gwen@hooligan0.net>
 *          Raude-Morvan Damien (DrazziB) <drazzib@drazzib.com>
 *
 * Histoire:
 *  * 10/01/2006 DrazziB : Correction pour compatibilite PHP5 (la methode
 *      get_class() est desormais sensible a la casse).
 *  * 20/10/2005 Hooligan0 : Ajout du necessaire pour appeler une base mysql
 *      Ce support ne garanti pas que l'appli soit compatible avec mysql !!
 *  * 18/10/2005 Hooligan0 : Abandon du module MDB, utilisation des fonctions
 *      postgresql natives de php.
 *  * 14/07/2005 Hooligan0 : Deplacement des fonctions "Config*" vers la classe
 *      Gasell. Correction de la methode Select.
 */

define('ERR_BDD_connect_01',          10101);
define('ERR_BDD_TableCreate_01',      10301);
define('ERR_BDD_TableDrop_01',        10401);
define('ERR_BDD_TableAddCol_01',      10501);
define('ERR_BDD_TableDropCol_01',     10601);
define('ERR_BDD_wrapper_badtype',     10701);
define('ERR_BDD_AddConstraint_01',    10801);

/**
 * @class Bdd
 *
 * @brief Classe permettant l'abstraction des serveur de base de donnees
 * en permettant l'acces a des fonctions de haut niveau
 */
class Bdd
{
	
	/** ============================================================= **
	 **                     VARIABLES D'INSTANCE                      **
	 ** ============================================================= **/

	 var $dbType;      // le type du serveur de base de donnees a utiliser
	 var $dbUser;      // le login de connexion
	 var $dbPass;      // le secret de connexion
	 var $dbHost;      // l'adresse reseau du SGBD
	 var $dbDb;        // le nom de la base a considerer
	 var $dbPrefix;    // le prefix a utiliser pour les tables
	 var $dbLink;      // le lien vers la connexion

	/** ============================================================= **
	 **                       METHODES PRIVEES                        **
	 ** ============================================================= **/

	/**
	 * @method _request
	 *
	 * @brief Fonction permettant l'envoie d'une requete SQL
	 * a la base de donnees. Fonction de base niveau a ne pas
	 * utiliser directement. Passer par les autres methods publiques
	 *
	 * @param $sql     sql_request    la requete SQL a executer
	 * @param $options indexed_array  tableau des options qui peuvent etre
	 * passees parmis lesquelles :
	 *   - return_res : place a true, renvoie un tableau avec des donnees
	 *      selectionnees dans la base
	 *   - auto_index : renvoie la valeur de l'index genere automatiquement 
	 *      par le SGBD
	 *
	 * @return boolean  flag indiquant si la requete s'est bien deroulee
	 * @return array    donnees renvoyees par la base
	 */
	function _request($sql, $options = array())
	{
		$result = false;
		/* on wrappe l'execution en fonction du type de le base */
		switch($this->dbType)
		{
			case "pgsql":
				$result = $this->_pg_request($sql, $options); 
				break;

			default:
				$msg = "Type de base de donn&eacute;es ".$this->dbType.
					" non support&eacute;";
				$this->_sendError(ERR_BDD_wrapper_badtype, $msg, "_request");
		}

		/* on renvoie le resultat de l'execution */
		return $result;
	}


	/**
	 * @method _pg_request
	 *
	 * @brief Fonction permettant l'envoie d'une requete SQL
	 *  a une base de donnees de type PostgreSQL. 
	 *  Fonction de base niveau a ne pas utiliser du tout.
	 *
	 * @param $sql     sql_request    la requete SQL a executer
	 * @param $options indexed_array  tableau des options qui peuvent etre
	 * passees parmis lesquelles :
	 *   - return_res : place a true, renvoie un tableau avec des donnees
	 *      selectionnees dans la base
	 *   - auto_index : renvoie la valeur de l'index genere automatiquement 
	 *      par le SGBD
	 *
	 * @return boolean  flag indiquant si la requete s'est bien deroulee
	 * @return array    donnees renvoyees par la base
	 */
	function _pg_request($sql, $options)
	{
		/* connexion a la base si necessaire */
		if($this->_pg_connect(true)) 
		{
			/* la requete a-t-elle reussie ? */
			if( ($res = pg_query($this->dbLink, $sql)) == false) 
				return false;
			else {
				/* par defaut on renvoie true */
				$return = true;

				/* est-ce qu'on attend un tableau de valeurs ? */
				if(isset($options["return_res"]) && $options["return_res"] == true) 
				{
					$return = array();
					while($row = pg_fetch_row($res))
						array_push($return, $row);
				}

				/*  est-ce qu'on attend une valeur d'index ? */
				if(isset($options["auto_index"]) && $options["auto_index"] == true)
				{
					$return = pg_last_oid($res);
				}
				
				/* on renvoie le resultat */
				return $return;
			}
		} else {
			/* la connexion a echouee */
			return false;
		}
	}


	/**
	 * @method _connect
	 *
	 * @brief fonction permettant la connexion a la base
	 *
	 * @noparam
	 *
	 * @result handle  identifiant de la connexion
	 * @result boolean false, echec de connexion
	 **/
	function _connect() 
	{
		/* on wrappe en fonction du type de serveur */
		switch($this->dbType)
		{
			case "pgsql":
				return $this->_pg_connect();

			default:
				$msg = "Type de base de donn&eacute;es ".$this->dbType.
					" non support&eacute;";
				$this->_sendError(ERR_BDD_wrapper_badtype, $msg, "_connect");
				return false;
		}
	}


	/**
	 * @method _pg_connect
	 *
	 * @brief fonction permettant la connexion a une base 
	 *  de donnees de type PostgreSQL
	 *
	 * @param $report boolean flag indiquant si l'on doit
	 *   faire un rapport d'erreur en cas d'echec de connexion
	 *
	 * @return boolean flag indiquant le succes de la connexion
	 **/
	function _pg_connect($report = true) 
	{
                /* Dans PHP5, la fonction get_class respecte maintenant la casse du nom de la classe
                   on utilise donc la fonction strtolower() afin de s'assurer que la comparaison
                   fonctionne sous PHP4 et PHP5 */
		if ((! isset($this)) || (strtolower(get_class($this)) != 'bdd'))
			return false;
		elseif ($this->_pg_isConnected())
			return true;
		else {
			$connHost = "host="     . $this->dbHost . " ";
			$connUser = "user="     . $this->dbUser . " ";
			$connPass = "password=" . $this->dbPass . " ";
			$conndb   = "dbname="   . $this->dbDb   . " ";

			$connString = $connHost . $connUser . $connPass . $conndb;

			/* Connection a la base                        */
			$this->dbLink = pg_connect($connString);

			/* Si une erreur est survenue                  */
			if (! $this->_pg_isConnected()) 
			{
				if ($report == 1) 
				{
					/* manage error */
					$msg = "Impossible de se connecter";
					$this->_sendError(ERR_BDD_connect_01, $msg, "_pg_connect");
				}
				return false;
			} else {
				return true;
			}
		}
	}


	/**
	 * @method _pg_isConnected
	 *
	 * @brief Fonction permettant de savoir si le lien
	 * vers un serveur PostgreSQL est valide
	 *
	 * @result boolean resultat du test
	 **/
	function _pg_isConnected() 
	{
		/* flag de connexion */
		$flg_c = ($this->dbLink !== NULL);
		/* utilisation de pg_connection_status si disponible */
		if (function_exists('pg_connection_status'))
			$flg_c = (pg_connection_status($this->dbLink) === 
				PGSQL_CONNECTION_OK);

		/* return the result */
		return $flg_c;
	}


	/**
	 * @method _sendError
	 * 
	 * @brief renvoie une erreur au core de GASELL
	 *
	 * @param $code le code de l'erreur
	 * @param $msg  le message d'erreur
	 * @param $err  message d'erreur complementaire
	 **/
	function _sendError($code, $msg, $err)
	{
		if(include(GASELL_BASE . '/lib/Exceptions.php')) 
		{
			Except::throwError("setup/Bdd.php", $msg, 
									array(
										"errcode" => $code,
										"errmsg"  => $msg,
										"errmsg2" => $err) );
		} else {
			/* Except not reachable, handle error manually */
			echo "<b>".$code."</b> : ".$msg."(".$err.")<br />";
		}
	}


	/** ============================================================= **
	 **                       METHODES PUBLICS                        **
	 ** ============================================================= **/


	/**
	 * @method Bdd (constructeur)
	 *
	 * @brief constructeur de la classe. Prend en parametre
	 * une url de connexion ou bien prend les donnees dans la
	 * configuration de GASELL
	 *
	 * @param $url url_db url de connexion a la base de donnees
	 * de la forme type://login:pass@address/database
	 *
	 * @noreturn
	 **/
	function Bdd($url = "")
	{
		/* est-ce que les infos sont passees en parametre ? */
		if(isset($url) && !empty($url) && (parse_url($url) != false)) 
		{
			/* recuperation des infos de connexion */
			$data = parse_url($url);

			/* chargement des donnees de connexion */
			$this->dbType = $data['scheme'];
			$this->dbUser = $data['user'];
			$this->dbPass = $data['pass'];
			$this->dbHost = $data['host'];

			/* for the database, remove the starting slash */
			if(preg_match("/^\/(.+)/", $data['path'], $matches)) 
				$this->dbDb = $matches[1];
			else
				$this->dbDb = $data['path'];
		} else {
			/* chargement des donnees de connexion depuis la conf */
			global $conf;
			$this->dbType = $conf['bdd']['type'];
			$this->dbUser = $conf['bdd']['user'];
			$this->dbPass = $conf['bdd']['pass'];
			$this->dbHost = $conf['bdd']['host'];
			$this->dbDb = $conf['bdd']['db'];
		}
	}


	/**
	 * @method isConnected
	 *
	 * @brief fonction determinant si la connexion
	 *  est valide ou non
	 *
	 * @result boolean true si la connexion est valide
	 **/
	function isConnected() 
	{
		/* on wrappe */
		switch($this->dbType) 
		{
			case "pgsql":
				return $this->_pg_isConnected();

			default:
				$msg = "Type de base de donn&eacute;es ".$this->dbType.
					" non support&eacute;";
				$this->_sendError(ERR_BDD_wrapper_badtype, $msg, "_connect");
				return false;
		}
	}


	/**
	 * @method lastError
	 *
	 * @brief fonction qui permet de retourner la derniere
	 *  erreure rencontree pour la connexion courante
	 *
	 * @param $referrer function_name the function which has
	 *  called this method
	 *
	 * @result string the message corresponding to the error
	 **/
	function lastError($referrer) 
	{
		/* check if we are connecter */
		if($this->isConnected()) 
		{
			/* wrappe the last error function */
			switch ($this->dbType) 
			{
				case "pgsql":
					return pg_last_error();
					break;

				default:
					$msg = "Type de base de donn&eacute;es ".$this->dbType.
						" non support&eacute;";
					$this->_sendError(ERR_BDD_wrapper_badtype, $msg, 
						"lastError referrer : ".$referrer);
					return false;
			}
		} else {
			return "Not connected to the database";
		}
	}


	/**
	 * @method reconnectDB
	 *
	 * @brief fonction qui permet la reconnexion de l'objet
	 *  a une base de donnees en parametre
	 *
	 * @param $name db_name le nom de la base
	 *
	 * @result boolean flag indiquant le succes de la reconnexion
	 **/
	function reconnectDB($name)
	{
		$this->dbDb = $name;
		$this->dbLink = false;
		$this->_connect();

		return $this->isConnected();
	}


	/**
	 * @method Insert
	 *
	 * @brief fonction permettant l'ajout de donnees dans la base
	 *
	 * @param $table     string        nom de la table dans laquelle 
	 *    realiser l'ajout
	 * @param $cols      indexed_array tableau dont l'index est le nom 
	 *    de la colonne et la valeur la valeur a renseigner pour cette colonne
	 * @param $retserial boolean       flag indiquant si un index automatique
	 *    va etre generee par le sgbd ou non
	 *
	 * @return boolean renvoie si l'insertion s'est bien passee
	 * @return mixed   renvoie l'index automatique genere
	 **/
	function Insert($table, $cols, $retserial = false)
	{
		/* pas d'appel sans instanciation */
		if (! isset($this) )
			die("MAIN ERROR - Bdd::Insert class not instanciated");

		if( strtoupper(get_class($this)) != 'BDD')
			die("MAIN ERROR - Bdd::Insert wrong class");

		/* initialisation */
		$q_cols = "";
		$q_vals = "";

		/* Pour chacune des valeurs a prendre en compte dans l'ajout     */
		foreach($cols as $clef => $val)
		{
			/* Si l'option "Magic Quotes" de php n'est pas activee */
			if (get_magic_quotes_gpc() != 1)
				/* conversion du format (sequences d'echapement) */
				$val = addslashes($val);

			/* Ajout du nom de la colone dans la requette            */
			$q_cols .= ($q_cols == "") ? $clef : ", $clef";

			/* Ajout de la valeurs de cette colone dans la requette  */
			$q_vals .= ($q_vals == "") ? "'".pg_escape_string($val)."'" 
				: ",'".pg_escape_string($val)."'";
		}

		/* Construction de la requette SQL apartir des colones et valeurs */
		$query = "INSERT INTO " . $table . " (" . $q_cols . ") ";
		$query .= "VALUES (" . $q_vals . ")";

		/* on envoie la requete au serveur */
		$result = $this->_request($query, array("auto_index" => $retserial));

		return($result);
	}


	/** ---------------------------------------------------------------- **
	 **         METHODES EN RAPPORT AVEC LES TABLES DE LA BASE           **
	 ** ---------------------------------------------------------------- **/

	/**
	 * @method TableCreate
	 *
	 * @brief fonction permettant de crer une table dans
	 * la base de donnees.
	 *
	 * @param $nom  string        le nom de la table
	 * @param $cols indexed_array tableau dont l'index est le nom de
	 *  la colonne a creer et le contenu son type
	 *
	 * @return boolean true est renvoye si tout s'est bien passe, false
	 * dans les autres cas.
	 **/
	function TableCreate($nom, $cols)
	{
		/* verification des parametres contre d'eventuels SQL injection */
		if(preg_match('/[\',;]/', $nom)) 
		{
			echo "<b>Le nom de la table est trop risqu&eacute; !</b><br />";
			$result = false;
		} else {
			/* Mise a zero du compteur de colones de la table */
			$nbparam = 0;

			/* Construction de la requette SQL de creation de la table */
			$query = 'CREATE TABLE ' . $nom . ' (';

			foreach($cols as $name => $type)
			{
				/* Ajoute 1 colone dans la requette de creation de la table */
				$query = $query . $name . ' ' . $type;
				/* Incremente le compteur de colones                        */
				$nbparam++;
				/* separation des colonnes sauf pour la derniere */
				if($nbparam < count($cols))
					$query .= ", ";
			}

			/* La requette SQL est fermee apres la declaration des colones   */
			$query = $query . ')';

			/* envoie de la requete au serveur */
			if( ($result = $this->_request($query)) == false)
				return false;
			else
				return $result;
		}
	} // END TableCreate


	/**
	 * @method TableDrop
	 *
	 * @brief fonction permettant de supprimer une table
	 *
	 * @param $nom     string   le nom de la table
	 * @param $cascade boolean  flag indiquant si la suppression
	 *  se fait en cascade ou non
	 *
	 * @return boolean true est renvoye si tout s'est bien passe, false
	 * dans les autres cas.
	 **/
	function TableDrop($nom, $cascade = false) 
	{
		/* Construction de la requette SQL de creation de la table  */
		$query  = 'DROP TABLE ' . $nom;
		$query .= ($cascade) ? ' CASCADE' : '';

		/* envoi de la requete */
		if ( ($result = $this->_request($query)) == false ) 
		{
			/* manage error */
			$msg = "Impossible de supprimer la table $nom";
			$this->_sendError(ERR_BDD_TableDrop_01, $msg, 
							$this->lastError("TableDrop"));
		}

		return $result;
	}


	/** ---------------------------------------------------------------- **
	 **            METHODES EN RAPPORT AVEC LES CONTRAINTES              **
	 ** ---------------------------------------------------------------- **/


	/**
	 * @methode AddConstraint
	 *
	 * @brief fonction permettant d'ajouter une contrainte
	 *  a une table de la base de donnees
	 *
	 * @param $nom    string le nom de la table concernee
	 * @param $contr  string expression de la contrainte
	 *
	 * @return boolean flag de succes de l'ajout de la contrainte
	 **/
	function AddConstraint($nom, $contr)
	{
		/* construction de la requete SQL */
		$sql = "ALTER TABLE ".$nom." ADD ".$contr;

		/* envoi de la requete */
		$result = $this->_request($sql);
		if (! $result) 
		{
			/* manage error */
			$msg  = "Impossible d'ajouter la contrainte <b>$contr</b>";
			$msf .= " &agrave; la table $nom";
			$this->_sendError(ERR_BDD_AddConstraint_01, $msg,
							$this->lastError("AddConstraint"));
		}

		return $result;
	}
}

/* EOF */
?>
